\ tft-bmp.txt
\ Part of pixelmaler-v0.1 for Mecrisp-Quintus - A native code Forth implementation for RISC-V
\ Copyright (C) 2018  Matthias Koch
\ display longan nano
\ MB March 2020


\ This originates from source-file for mecrisp-quintus.
\ To fit to gforth emulator.fs it was rewritten but many words
\ are reused. That leads to some unlogical definitions.
\ especially bmp-line

\ MB April 2020


\ Examples how to read in and how to display *.BMP pictures.
\ These picture muste be in a special format. With GIMP it
\ is easy to convert nearly every picture to this format:
\ Load the picture into GIMP, cut or shrink it to a size that
\ fits the needs of the display i.e. width/hights < 160/80,
\ export it: /File/Export As ... . Type in picture name with
\ extention BMP. A second dialog will open. Now choose
\ /Advanced Options/16 Bit/R5 G6 B5. Klick 'Export'
\ Done!

\ For a first attempt you may use the two pictures in this
\ tft-suite: dragon36.bmp or flower.bmp
\ The internal words are self-explainig (I hope).

\ Variables

512 constant BpS

0 Variable bmp-header-len \ has to be a multiple of 2
0 Variable bmp-width
0 Variable bmp-height
0 Variable bmp-len
0 Variable bmp-compressed
0 Variable bmp-r-mask
0 Variable bmp-g-mask
0 Variable bmp-b-mask
0 Variable bmp-pointer
0 Variable bmp-sector
0 Variable bmp-start-sector
0 Variable bmp-fill

0 Value bmp-file
0 Value bmp-file-len


\ I'll use slurp-file

Create bmp-file-tab 8 cells 1 + allot \ room for 4 bmp-files
 bmp-file-tab 8 cells 1 + erase

: bmp-file! ( addr len -- )
    bmp-file-tab count
    4 mod 2 cells * +
    2!
    bmp-file-tab c@ 1+ bmp-file-tab c!
;

: get-bmp-file ( n -- )
    2 cells * 1+
    bmp-file-tab +
    2@
    to bmp-file-len
    to bmp-file
;



: read-bmp-header ( -- )
    bmp-file 10 + l@ bmp-header-len !
    bmp-file 18 + l@ bmp-width !
    bmp-file 22 + l@ bmp-height !
    bmp-file 34 + l@ bmp-len !
    bmp-file 30 + l@ bmp-compressed !
    bmp-file 54 + l@ bmp-r-mask !
    bmp-file 58 + l@ bmp-g-mask !
    bmp-file 62 + l@ bmp-b-mask !
;

: bmp-match? ( -- flag )
    bmp-width @ tft-width @  <
    bmp-height @ tft-height @ <
    bmp-compressed @ 3 =
    bmp-r-mask @ $F800 =
    bmp-g-mask @ $7e0 =
    bmp-b-mask @ $1F =
    or or or or or
; 



\ : bmp-init-sectors ( block -- )
\     dup bmp-start-sector !
\     0 bmp-sector !
\     sd-read
\ ;

\ \ write one word (16bit) from SD-card to the display

\ : @bmp>tft ( -- )
\     bmp-pointer @
\     BpS /mod
\     dup bmp-sector @ = 
\     IF drop
\     ELSE
\         dup bmp-sector !
\         bmp-start-sector @ +
\         sd-read 
\     THEN
\     sd.buf + h@
\     2bytes>tft
\     2 bmp-pointer +!
\ ;




\ set the area that will be filled with the BMP datas

: bmp-set-shape ( -- )
    tft-width @ bmp-width @ - 2/   \ center
    tft-height @ bmp-height @ - 2/ \ center
    bmp-width @ 1-
    bmp-height @ 1- xywh+
    set-shape
;


\ primitve 'file' handling. Look if the given sector is
\ a BMP header, adjust file pointer.

: bmp-open  ( addr c -- )
    slurp-file
    bmp-file!
;

: validate ( n -- )
    get-bmp-file
    read-bmp-header
    bmp-match?
    IF  4 bmp-width @ 4 mod - 4 mod bmp-fill !
        bmp-set-shape
        bmp-header-len @ bmp-pointer !
    ELSE 
        ." BMP-Header doesn't match!"
        quit
    THEN
;

: 5b>8b ( n -- n )
    %11111 and \ just to be shure
    $FF swap %11111 */
;

: 6b>8b ( n -- n )
    %111111 and \ just to be shure
    $FF swap %111111 */
;


: rgb16>rgba32 ( n -- n )
    dup %11111 and 5b>8b 8 lshift 
    swap 5 rshift dup %111111 and 6b>8b 16 lshift
    swap 6 rshift     %11111  and 5b>8b 24 lshift  \ b g r
    or or $FF or \ rgba
;

: 4aligned ( n -- n )
    4 /mod swap IF 1+ THEN 4 *
;

: back2lines ( -- )
    bmp-width @ 4aligned 2* 2* negate
    bmp-pointer +!
;    

\ read-in one line of BMP picture
: bmp-line ( -- )
    bmp-width @ 0
    DO
        bmp-file bmp-pointer @ +
        w@ dup rgb16>rgba32 t-color ! 2bytes>tft
        2 bmp-pointer +!
    LOOP
    bmp-fill @ 0
    ?DO
        2 bmp-pointer +!
    LOOP
    back2lines
;

\ Pictures smaller than the display will be centered.
\ As we read in the picture from the start i.e. bottom
\ line we have to change the orientation of our display.
\ This is done via the word mirror-v.
\ Make shure that now tft-command is written after invoking
\ 'write-memory'. The sequence ' ... write-memory ...
\ ... mirror-v ... wont work.

: >last-line ( -- )
    bmp-file-len
    bmp-width @  4aligned 2*
    - bmp-pointer !
;


: bmp-show ( n -- )
    dup get-bmp-file
    validate
    >last-line
    write-memory
    bmp-height @ 0
    DO
        bmp-line
    LOOP
    update-memimage
;

s" drache36.bmp" bmp-open
s" flower.bmp" bmp-open


: bmp-demo
    ." Endless Diashow!"
    BEGIN
        0 bmp-show
        2000 ms
        1 bmp-show
        2000 ms
        key?
    UNTIL 
;
