\ turtle.fs
\ Part of pixelmaler-v0.1 for Mecrisp-Quintus - A native code Forth implementation for RISC-V
\ Copyright (C) 2018  Matthias Koch
\ display longan nano
\ MB March 2020


\ This is the original source-file for mecrisp-quintus.
\ To fit to gforth emulator.fs some changes were made:
\ added new word degr>rad
\ changed fixpoint t-sin, t-cos, t-x, t-y to floating point (fvalue)
\ changed words 'mv-to', 'home', 'turn', #fd-y', 'fd-x', and
\ 'fd' to internal use of floating point numbers
\ MB April 2020



\ simple turtle graphics
\ It uses the fixpoint math from
\ common/fixpt-math-lib.fs
\ including some triogonemetry especially sin cos.
\ Calculating the position of the turtle with fixpoint math has
\ the great advantage to avoid rounding faults.

0e fValue t-sin
0e fValue t-cos
0e fValue t-x
0e fValue t-y

0 Variable t-orientation 

0 Variable pen 
0 Variable t-fg-col
0 Variable t-bg-col

: .t-param ( -- )
    cr ."         sin: " t-sin f.
    cr ."           x: " t-x   f.
    cr ."         cos: " t-cos f.
    cr ."           y: " t-y   f.
    cr ." orientation: " t-orientation @ .
;

: pencolor ( -- )
    color @ t-fg-col !
;

: groundcolor ( -- )
    color @ t-bg-col !
;


: pu ( -- )
    false pen !
;

: pd ( -- )
    true pen !
;

: mv-to ( x y -- )
    pen @
    IF
        t-x f>d drop t-y f>d drop
        2over
        line
    THEN
    s>f ['] t-y fValue-to 
    s>f ['] t-x fValue-to 
;

: erase ( -- )
    t-bg-col @
    color !
; 


: degr>rad ( n -- ) ( F: -- r ) \ signed hundredth of degrees i.e. 45° --> 4500 
    pi fdup f+
    s>f 100e f/ f*
    360e  f/
;


: home \ home
    0 t-orientation !
    0e ['] t-sin fValue-to
    1e ['] t-cos fValue-to
    tft-width @  2/ s>f ['] t-x fValue-to 
    tft-height @ 2/ s>f ['] t-y fValue-to
;

: t-page ( -- )
    t-bg-col @ color !
    tft-clear
    t-fg-col @ color !
    home
;


: turn ( n -- ) \ signed hundredth of degrees i.e. 45° --> 4500
    t-orientation @ + 36000 mod dup t-orientation !
    degr>rad 
    fsincos
    ['] t-cos fValue-to 
    ['] t-sin fValue-to 
;

: tl ( n -- ) \ turn left
    turn
;
: tr ( n -- ) \ turn right
    negate turn
;


: fd-y ( n -- y0 y1 )
    s>f t-cos f* 
    t-y fdup f>d drop f+
    fdup ['] t-y fValue-to
    f>d drop
;

: fd-x ( n -- x0 x1 )
    s>f t-sin f*
    t-x fdup f>d drop f+
    fdup ['] t-x fValue-to
    f>d drop
;

    
: fd ( n -- )
    dup fd-x
    rot fd-y
    rot swap
    pen @
    IF
        line
    ELSE
        2drop 2drop
    THEN
;

: backward ( n -- )
    18000 turn
    fd
    18000 turn
; 

: spiral ( n -- ) \ degrees 
    home
    360 over / 0
    DO
        dup 100 * turn
        4 0 DO 9000 turn 30 fd LOOP
    LOOP
    drop
;

: turtle-demo ( -- )
    white groundcolor
    blue pencolor
    t-page pd
    10 spiral
;
